@@ -27,7 +27,6 @@ static int x_error_handler(Display *dpy, XErrorEvent *xevent);
 static void key_grab(Display *dpy, guint keyval);
 static void key_ungrab(Display *dpy, guint keyval);
 #ifdef NOTIFY
-static void notifier_clicked(NotifyNotification *notify, gchar *actionID, gpointer user_data);
 static void notification_toggled(GObject *obj, gpointer user_data);
 #endif
 static gchar* strip_invalid_edges(gchar *selection);
@@ -40,7 +39,7 @@ static void quit_activate(GObject *obj, gpointer user_data);
 static guint8 get_frequency(guint sense_count);
 static void relatives_clear_all(GtkBuilder *gui_builder);
 static void antonyms_load(GSList *antonyms, GtkBuilder *gui_builder);
-static guint8 append_term(gchar *target, gchar *term, gboolean is_heading);
+static guint16 append_term(gchar *target, gchar *term, gboolean is_heading);
 static void build_tree(GNode *node, GtkTreeStore *tree_store, GtkTreeIter *parent_iter);
 static void trees_load(GSList *properties, GtkBuilder *gui_builder, WNIRequestFlags id);
 static void list_relatives_load(GSList *properties, GtkBuilder *gui_builder, WNIRequestFlags id);
@@ -73,7 +72,10 @@ static void save_preferences();
 static void about_email_hook(GtkAboutDialog *about_dialog, const gchar *link, gpointer user_data);
 static void about_url_hook(GtkAboutDialog *about_dialog, const gchar *link, gpointer user_data);
 static void destructor(Display *dpy, guint keyval, GtkBuilder *gui_builder);
-
+static gchar* wildmat_to_regex(gchar *wildmat);
+static void set_regex_results(gchar *wildmat_exp, GtkBuilder *gui_builder);
+static gboolean is_wildmat_expr();
+static gboolean wordnet_terms_load(GtkBuilder *gui_builder);
 
 
 static int x_error_handler(Display *dpy, XErrorEvent *xevent)
@@ -134,20 +136,6 @@ static void key_ungrab(Display *dpy, guint keyval)
 }
 
 #ifdef NOTIFY
-static void notifier_clicked(NotifyNotification *notify, gchar *actionID, gpointer user_data)
-{
-	GtkBuilder *gui_builder = GTK_BUILDER(user_data);
-	GtkWindow *window = GTK_WINDOW(gtk_builder_get_object(gui_builder, WINDOW_MAIN));
-	GtkWidget *combo_query = GTK_WIDGET(gtk_builder_get_object(gui_builder, COMBO_QUERY));
-
-	if(0 == wni_strcmp0(actionID, "lookup"))
-	{
-		gtk_window_present(window);
-		gtk_widget_grab_focus(GTK_WIDGET(combo_query));
-	}
-
-}
-
 /*
 	This will be called for both notify check pop-up menu and notify tool bar button
 	Hence the first argument is GObject. Since both have the propery "active" as its
@@ -263,6 +251,12 @@ static void status_icon_activate(GtkStatusIcon *status_icon, gpointer user_data)
 		gtk_widget_hide(GTK_WIDGET(window));
 	else
 	{
+
+#ifdef NOTIFY
+		// close notifications, if any
+		notify_notification_close(notifier, NULL);
+#endif
+
 		// Code to watch clipboard when popping up
 		// Should be enabled when the 'Watch Clipboard' settings is put up in Pref. window
 		/*xe.type = KeyPress;
@@ -305,11 +299,11 @@ static void about_activate(GtkToolButton *menu_item, gpointer user_data)
 	GtkWidget *about_dialog = gtk_about_dialog_new();
 
 	if(hotkey_index != -1)
-		about_comments = g_strdup_printf("%s\r\n\r\n%s%c", STRING_ABOUT, ABOUT_HOTKEY_SET, hot_key_vals[hotkey_index - 1] - 32);
+		about_comments = g_strdup_printf("%s\r\n\r\n%s%c\r\n", STRING_ABOUT, ABOUT_HOTKEY_SET, hot_key_vals[hotkey_index - 1] - 32);
 	else
 		about_comments = STRING_ABOUT;
-	
-	g_object_set(about_dialog, "license", STRING_LICENCE, "copyright", STRING_COPYRIGHT, 
+
+	g_object_set(G_OBJECT(about_dialog), "license", STRING_LICENCE, "copyright", STRING_COPYRIGHT, 
 	"comments", about_comments, "authors", strv_authors, "version", PACKAGE_VERSION, 
 	"wrap-license", TRUE, "website-label", STRING_WEBSITE_LABEL, "website", STRING_WEBSITE, NULL);
 
@@ -455,9 +449,9 @@ static void antonyms_load(GSList *antonyms, GtkBuilder *gui_builder)
 	gtk_tree_view_set_headers_visible(tree_antonyms, has_indirect);
 }
 
-static guint8 append_term(gchar *target, gchar *term, gboolean is_heading)
+static guint16 append_term(gchar *target, gchar *term, gboolean is_heading)
 {
-	guint8 i = 0;
+	guint16 i = 0;
 
 	while(term[i] != '\0')
 	{
@@ -468,7 +462,7 @@ static guint8 append_term(gchar *target, gchar *term, gboolean is_heading)
 	++target;
 	*target = ' ';
 
-	return i + 2;
+	return (i + 2);
 }
 
 static void build_tree(GNode *node, GtkTreeStore *tree_store, GtkTreeIter *parent_iter)
@@ -477,7 +471,7 @@ static void build_tree(GNode *node, GtkTreeStore *tree_store, GtkTreeIter *paren
 	GSList *temp_list = NULL;
 	WNIImplication *imp = NULL;
 	GtkTreeIter tree_iter = {0};
-	guint8 i = 0;
+	guint32 i = 0;
 	//guint8 type_no = 0;
 	gchar terms[MAX_CONCAT_STR] = "";
 
@@ -530,7 +524,7 @@ static void trees_load(GSList *properties, GtkBuilder *gui_builder, WNIRequestFl
 	WNIImplication *imp = NULL;
 	GSList *temp_list = NULL;
 	gchar terms[MAX_CONCAT_STR] = "";
-	guint8 i = 0;
+	guint16 i = 0;
 	//guint8 type_no = 0;
 	
 	if(!advanced_mode && WORDNET_INTERFACE_HYPERNYMS == id) return;
@@ -836,12 +830,142 @@ static void relatives_load(GtkBuilder *gui_builder, gboolean reset_tabs)
 	// instead of invalidate rect redraw, we just toggle visibility
 	gtk_widget_show(expander);
 
-	// not required in all systems. At home (with compiz) it works perfectly fine without this code
+	// This is not required in all systems. With compiz, it works perfectly fine without this code.
 	// redraw the window after visibility changes
 	//if(GTK_WIDGET(gtk_builder_get_object(gui_builder, EXPANDER))->window != NULL)		// if its NULL, it means window is invisible, needn't redraw
 		//gdk_window_invalidate_rect(GTK_WIDGET(gtk_builder_get_object(gui_builder, EXPANDER))->window, NULL, TRUE);
 }
 
+static gchar* wildmat_to_regex(gchar *wildmat)
+{
+	guint16 i = 0;
+	gchar ch = 0;
+	GString *regex = g_string_new("^");
+
+	// Conversion Rules are simple at this point.
+	// 1. To get the proper term from start to end, make sure the beginning and end of the regex is ^ and $ respectively
+	// 2. E.g. in 'a*b', * is applied to a in regex, while in wildmat, 'a' & 'b' is sure included and * is anything b/w them
+	// so in regex make it as 'a.*b', which will make * applied to . (dot means any char. except newline in regex)
+	// 3. A dot/period in regex acts as a Joker (?) in wildmat, so make a direct conv.
+	// 4. Other chars go in as such, its the same in both for stuff like [r|m|s]{m,n} or [a-m]+, etc.
+
+	while((ch = wildmat[i++]) != '\0')
+	{
+		if(ch == '*')
+			regex = g_string_append(regex, ".*");
+		else if(ch == '?')
+			regex = g_string_append_c(regex, '.');
+		else
+			regex = g_string_append_c(regex, ch);
+		
+	}
+	
+	regex = g_string_append_c(regex, '$');
+	regex = g_string_append_c(regex, '\0');
+
+	return g_string_free(regex, FALSE);
+}
+
+static void set_regex_results(gchar *wildmat_exp, GtkBuilder *gui_builder)
+{
+	gchar *regex_pattern = NULL, *lemma = NULL;
+	GRegex *regex = NULL;
+	GMatchInfo *match_info = NULL;
+	guint32 count = 0;
+	GtkTextView *text_view = NULL;
+	GtkTextBuffer *text_buffer = NULL;
+	GtkTextIter cur = {0};
+	GtkStatusbar *status_bar = GTK_STATUSBAR(gtk_builder_get_object(gui_builder, STATUSBAR));
+	gchar status_msg[MAX_STATUS_MSG] = "";
+
+
+	// convert the wilmat expr. to PERL regex
+	if(wildmat_exp)
+		regex_pattern = wildmat_to_regex(wildmat_exp);
+
+	if(regex_pattern)
+	{
+		text_view = GTK_TEXT_VIEW(gtk_builder_get_object(gui_builder, TEXT_VIEW_DEFINITIONS));
+		text_buffer = gtk_text_view_get_buffer(text_view);
+
+		// clear text and get the iter
+		gtk_text_buffer_set_text(text_buffer, "", -1);
+		gtk_text_buffer_get_start_iter(text_buffer, &cur);
+
+		// set heading that Regex mode is now in action
+		gtk_text_buffer_insert_with_tags_by_name(text_buffer, &cur, STR_REGEX_DETECTED, -1, TAG_LEMMA, NULL);
+		gtk_text_buffer_insert(text_buffer, &cur, NEW_LINE, -1);
+
+		// compile a GRegex
+		regex = g_regex_new(regex_pattern, G_REGEX_MULTILINE|G_REGEX_CASELESS|G_REGEX_UNGREEDY|G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
+
+		if(regex)
+		{
+			// do the actual regex matching and get the count
+			g_regex_match(regex, wordnet_terms->str, G_REGEX_MATCH_NOTEMPTY, &match_info);
+			count = g_match_info_get_match_count(match_info);
+
+			// make a header for the suggestions to follow
+			if(count > 0)
+			{
+				gtk_text_buffer_insert_with_tags_by_name(text_buffer, &cur, STR_SUGGEST_MATCHES, -1, TAG_MATCH, NULL);
+			}
+
+			// initialize count to 0 for the actual count to be displayed in the status bar
+			count = 0;
+
+			// insert the matches one by one
+			while(g_match_info_matches(match_info))
+			{
+
+				lemma = g_match_info_fetch(match_info, 0);
+				gtk_text_buffer_insert(text_buffer, &cur, NEW_LINE, -1);
+				gtk_text_buffer_insert_with_tags_by_name(text_buffer, &cur, lemma, -1, TAG_SUGGESTION, NULL);
+				g_free (lemma);
+				g_match_info_next (match_info, NULL);
+				count++;
+			}
+
+			// free all the initialised data
+			g_match_info_free (match_info);
+			g_regex_unref(regex);
+
+			// set the status bar accordingly with the count
+			gtk_statusbar_pop(status_bar, status_msg_context_id);
+			g_snprintf(status_msg, MAX_STATUS_MSG, STR_STATUS_REGEX, count, count > 0?STR_STATUS_LOOKUP_HINT:"");
+			status_msg_context_id = gtk_statusbar_get_context_id(status_bar, STATUS_DESC_REGEX_RESULTS);
+			gtk_statusbar_push(status_bar, status_msg_context_id, status_msg);
+		}
+
+		// if regex was invlaid (it would be NULL)
+		// or if the expr. fetched no results, set regex failed message
+		if(NULL == regex || 0 == count)
+		{
+			gtk_text_buffer_insert_with_tags_by_name(text_buffer, &cur, STR_REGEX_FAILED, -1, TAG_SUGGESTION, NULL);
+		}
+
+		g_free(regex_pattern);
+	}
+}
+
+static gboolean is_wildmat_expr(gchar *expr)
+{
+	guint16 i = 0;
+	gchar ch = 0;
+
+	if(expr)
+	{
+		// if the passed expr. has any of the char below deem it as a wildmat expr.
+		while((ch = expr[i++]) != '\0')
+		{
+			if(ch == '*' || ch == '?' || ch == '[' || ch == ']' || ch =='{' || ch == '}' || ch == '+')
+				return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 static void button_search_click(GtkButton *button, gpointer user_data)
 {
 	gchar *search_str = NULL, *lemma = NULL, **suggestions = NULL;
@@ -871,15 +995,69 @@ static void button_search_click(GtkButton *button, gpointer user_data)
 #endif
 
 	gboolean results_set = FALSE;
+	gchar *regex_text = NULL;
 
 	GtkStatusbar *status_bar = GTK_STATUSBAR(gtk_builder_get_object(gui_builder, STATUSBAR));
-	static guint msg_context_id = 0;
 	gchar status_msg[MAX_STATUS_MSG] = "";
 	guint16 total_results = 0;
 
+
+
 	text_view = GTK_TEXT_VIEW(gtk_builder_get_object(gui_builder, TEXT_VIEW_DEFINITIONS));
 	buffer = gtk_text_view_get_buffer(text_view);
 
+
+	// Check if the fed string is a wildmat expr.
+	// If true, call the regex mod. to set the results and return
+	regex_text = g_strstrip(gtk_combo_box_get_active_text(combo_query));
+	if((results_set = is_wildmat_expr(regex_text)))
+	{
+		// clear previously set relatives
+		relatives_clear_all(gui_builder);
+
+		if(wordnet_terms)
+		{
+			// set the status bar to inform that the search is on going
+			g_snprintf(status_msg, MAX_STATUS_MSG, STR_STATUS_SEARCHING);
+			status_msg_context_id = gtk_statusbar_get_context_id(status_bar, STATUS_DESC_REGEX_SEARCHING);
+			gtk_statusbar_push(status_bar, status_msg_context_id, status_msg);
+
+			// if visible, update the statusbar; the window will usually go on a freeze
+			// without updating the statusbar, until GDK is idle
+			// this call forces GDK to first redraw the statusbar and proceed
+			if(GTK_WIDGET_VISIBLE(window))
+				gdk_window_process_updates(((GtkWidget*)status_bar)->window, FALSE);
+
+			set_regex_results(regex_text, gui_builder);
+		}
+		else
+		{
+			// report that the search index (sense.index) is missing
+			g_snprintf(status_msg, MAX_STATUS_MSG, STR_STATUS_REGEX_FILE_MISSING);
+			status_msg_context_id = gtk_statusbar_get_context_id(status_bar, STATUS_DESC_REGEX_ERROR);
+			gtk_statusbar_push(status_bar, status_msg_context_id, status_msg);
+
+			gtk_text_buffer_set_text(buffer, "", -1);
+			gtk_text_buffer_get_end_iter(buffer, &cur);
+
+			lemma = g_strdup_printf(STR_REGEX_FILE_MISSING, SetSearchdir());
+			if(lemma)
+			{
+				gtk_text_buffer_insert_with_tags_by_name(buffer, &cur, lemma, -1, TAG_POS, NULL);
+
+				g_free(lemma);
+				lemma = NULL;
+			}
+		}
+	}
+
+	g_free(regex_text);
+
+	// If it was a regex and its results are now furnished, so return
+	if(results_set) return;
+
+
+	// from here on normal look-up starts
 	search_str = g_strstrip(strip_invalid_edges(gtk_combo_box_get_active_text(combo_query)));
 
 	if(search_str)
@@ -892,7 +1070,6 @@ static void button_search_click(GtkButton *button, gpointer user_data)
 		if(notifier)
 		{
 			notify_notification_close(notifier, NULL);
-			notify_notification_clear_actions(notifier);
 		}
 #endif
 
@@ -903,7 +1080,15 @@ static void button_search_click(GtkButton *button, gpointer user_data)
 
 		if((last_search != NULL && total_results != 0) || (total_results == 0 && !last_search_successful) || last_search == NULL)
 		{
-			gtk_statusbar_pop(status_bar, msg_context_id);
+			// set the status bar and update the window
+			gtk_statusbar_pop(status_bar, status_msg_context_id);
+			g_snprintf(status_msg, MAX_STATUS_MSG, STR_STATUS_SEARCHING);
+			status_msg_context_id = gtk_statusbar_get_context_id(status_bar, STATUS_DESC_REGEX_SEARCHING);
+			gtk_statusbar_push(status_bar, status_msg_context_id, status_msg);
+			
+			// if visible, repaint the statusbar after setting the status message, for it to get reflected
+			if(GTK_WIDGET_VISIBLE(window))
+				gdk_window_process_updates(((GtkWidget*)status_bar)->window, FALSE);
 
 			G_MESSAGE("'%s' requested from WNI!\n", search_str);
 
@@ -916,7 +1101,6 @@ static void button_search_click(GtkButton *button, gpointer user_data)
 			if(results)
 			{
 				total_results = 0;
-				msg_context_id = 0;
 
 				G_MESSAGE("Results successful!\n");
 
@@ -984,7 +1168,7 @@ static void button_search_click(GtkButton *button, gpointer user_data)
 						gtk_text_buffer_insert(buffer, &cur, NEW_LINE, -1);
 					}
 
-					msg_context_id++;
+					status_msg_context_id++;
 				}
 
 				// scroll to top code goes here
@@ -992,9 +1176,9 @@ static void button_search_click(GtkButton *button, gpointer user_data)
 				gtk_text_buffer_add_mark(buffer, freq_marker, &cur);
 				gtk_text_view_scroll_to_mark(text_view, freq_marker, 0, FALSE, 0, 0);*/
 
-				g_snprintf(status_msg, MAX_STATUS_MSG, STR_STATUS_QUERY_SUCCESS, total_results, msg_context_id);
-				msg_context_id = gtk_statusbar_get_context_id(status_bar, "search_successful");
-				gtk_statusbar_push(status_bar, msg_context_id, status_msg);
+				g_snprintf(status_msg, MAX_STATUS_MSG, STR_STATUS_QUERY_SUCCESS, total_results, status_msg_context_id);
+				status_msg_context_id = gtk_statusbar_get_context_id(status_bar, STATUS_DESC_SEARCH_SUCCESS);
+				gtk_statusbar_push(status_bar, status_msg_context_id, status_msg);
 
 				lemma = ((WNIDefinitionItem*)((WNIOverview*)((WNINym*)results->data)->data)->definitions_list->data)->lemma;
 
@@ -1054,7 +1238,6 @@ static void button_search_click(GtkButton *button, gpointer user_data)
 				definition = g_strconcat("<b><i>", partnames[((WNIDefinitionItem*)((WNIOverview*)((WNINym*)results->data)->data)->definitions_list->data)->pos], 
 				"</i></b>. ", ((WNIDefinition*)((WNIDefinitionItem*)((WNIOverview*)((WNINym*)results->data)->data)->definitions_list->data)->definitions->data)->definition, NULL);
 
-				notify_notification_add_action(notifier, "lookup", "Detailed Lookup", NOTIFY_ACTION_CALLBACK(notifier_clicked), gui_builder, NULL);
 				if(definition)
 				{
 					notify_notification_update(notifier, lemma, definition, "gtk-dialog-info");
@@ -1103,22 +1286,22 @@ static void button_search_click(GtkButton *button, gpointer user_data)
 						total_results = 0;
 					
 						gtk_text_buffer_insert(buffer, &cur, NEW_LINE, -1);
-						gtk_text_buffer_insert_with_tags_by_name(buffer, &cur, STR_SUGGEST_MATCHES, -1, TAG_SUGGEST, NULL);
-					
+						gtk_text_buffer_insert_with_tags_by_name(buffer, &cur, STR_SUGGEST_MATCHES, -1, TAG_MATCH, NULL);
+
 						while(suggestions[total_results])
 						{
 							gtk_text_buffer_insert(buffer, &cur, NEW_LINE, -1);
 							gtk_text_buffer_insert_with_tags_by_name(buffer, &cur, suggestions[total_results++], -1, TAG_SUGGESTION, NULL);
 						}
-					
+
 						g_strfreev(suggestions);
 					}
 				}
 
 				gtk_window_present(window);
 
-				msg_context_id = gtk_statusbar_get_context_id(status_bar, "search_failed");
-				gtk_statusbar_push(status_bar, msg_context_id, STR_STATUS_QUERY_FAILED);
+				status_msg_context_id = gtk_statusbar_get_context_id(status_bar, STATUS_DESC_SEARCH_FAILURE);
+				gtk_statusbar_push(status_bar, status_msg_context_id, STR_STATUS_QUERY_FAILED);
 #ifdef NOTIFY
 			}
 #endif
@@ -1179,18 +1362,102 @@ static gboolean text_view_button_pressed(GtkWidget *widget, GdkEventButton *even
 {
 	if(GDK_2BUTTON_PRESS == event->type)
 		was_double_click = TRUE;
+
 	return FALSE;
 }
 
 static gboolean text_view_button_released(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
-	// call query if it was a double click or a CTRL + selection
+	GtkBuilder *gui_builder = GTK_BUILDER(user_data);
+	GtkTextView *defn_text_view = GTK_TEXT_VIEW(gtk_builder_get_object(gui_builder, TEXT_VIEW_DEFINITIONS));
+	GtkTextBuffer *defn_text_buffer = gtk_text_view_get_buffer(defn_text_view);
+	GtkTextMark *insert_mark = NULL, *selection_mark = NULL;
+	GtkTextIter ins_iter = {0}, sel_iter = {0};
+	gchar *trial_text = NULL;
+	guint8 i = 0;
+	gboolean iter_move_success = FALSE;
+
+	// set the selection if it was a double click or a CTRL + mouse release
 	if(was_double_click || (GDK_CONTROL_MASK & event->state))
 	{
-		/* And request the "TARGETS" target for the primary selection */
+		// if it was a double click, try selecting diff. combinations
+		// for the sake of compound lemmas like 'work out', 'in a nutshell', etc.
+		if(was_double_click)
+		{
+			was_double_click = FALSE;
+
+			if(gtk_text_buffer_get_has_selection(defn_text_buffer))
+			{
+				// get the 'selection_bound' and 'insert' markers
+				selection_mark = gtk_text_buffer_get_selection_bound(defn_text_buffer);
+				insert_mark = gtk_text_buffer_get_mark(defn_text_buffer, "insert");
+
+				for(i = 0; ((i < 6) && (!iter_move_success)); i++)
+				{
+					// on every iteration of the loop, set the iters to the actual sel. marks
+					gtk_text_buffer_get_iter_at_mark(defn_text_buffer, &ins_iter, insert_mark);
+					gtk_text_buffer_get_iter_at_mark(defn_text_buffer, &sel_iter, selection_mark);
+
+					switch(i)
+					{
+						case 0:
+							// try selecting two words after the actual sel.
+							if(!gtk_text_iter_forward_visible_word_ends(&sel_iter, 2))
+								gtk_text_iter_forward_to_end(&sel_iter);
+							iter_move_success = TRUE;
+							break;
+						case 1:
+							// try selecting two words before the actual sel.
+							if(gtk_text_iter_backward_visible_word_starts(&ins_iter, 2))
+								iter_move_success = TRUE;
+							break;
+						case 2:
+							// try selecting one word before and after the actual sel.
+							if((gtk_text_iter_forward_visible_word_end(&sel_iter)) && (gtk_text_iter_backward_visible_word_start(&ins_iter)))
+								iter_move_success = TRUE;
+							break;
+						case 3:
+							// try selecting one word next to the actual sel.
+							if(!gtk_text_iter_forward_visible_word_end(&sel_iter))
+								gtk_text_iter_forward_to_end(&sel_iter);
+							iter_move_success = TRUE;
+							break;
+						case 4:
+							// try selecting one word before the actual sel.
+							if(gtk_text_iter_backward_visible_word_start(&ins_iter))
+								iter_move_success = TRUE;
+							break;
+						case 5:
+							// fallback to just the actual selection
+							iter_move_success = TRUE;
+							break;
+					}
+
+					if(iter_move_success)
+					{
+						// obtain the text for the set iters
+						trial_text = gtk_text_buffer_get_text(defn_text_buffer, &ins_iter, &sel_iter, FALSE);
+						
+						// search the trial selection in WordNet, on a successful look up iter_move_success is set
+						iter_move_success = wni_request_nyms(trial_text, NULL, 0, FALSE);
+
+						// free the obtained text
+						g_free(trial_text);
+						trial_text = NULL;
+					}
+				}
+
+
+				// set the 'insert' and 'selection_bound' markers respectivly to the newly placed iters
+				gtk_text_buffer_move_mark(defn_text_buffer, insert_mark, &ins_iter);
+				gtk_text_buffer_move_mark(defn_text_buffer, selection_mark, &sel_iter);
+			}
+		}
+
+		// request the "TARGETS" target for the primary selection
 		gtk_selection_convert(widget, GDK_SELECTION_PRIMARY, GDK_TARGET_STRING, event->time);
-		was_double_click = FALSE;
 	}
+
 	return FALSE;
 }
 
@@ -1535,60 +1802,6 @@ static void highlight_senses_from_relative_lists(WNIRequestFlags id, guint16 rel
 	}
 }
 
-/*
-static void highlight_senses_from_relative_trees(WNIRequestFlags id, guint16 relative_index, GtkTextView *text_view)
-{
-	GSList *wni_results = results;
-	GSList *properties_list = NULL, *term_list = NULL;
-	WNISynonymMapping *mapping = NULL;
-	GtkTextMark *mark = NULL;
-	GNode *node = NULL;
-	gboolean found = FALSE;
-	guint16 count = 0;
-
-	while(wni_results)
-	{
-		if(id == ((WNINym*)wni_results->data)->id)
-		{
-			properties_list = ((WNIProperties*)((WNINym*)wni_results->data)->data)->properties_list;
-
-			while(properties_list)
-			{
-				node = g_node_first_child((GNode*) properties_list->data);
-
-				while(node)
-				{
-					term_list = ((WNITreeList*) node->data)->word_list;
-					while(term_list)
-					{
-						count++;
-						if(count == relative_index + 1)
-						{
-							found = TRUE;
-							break;
-						}
-						term_list = g_slist_next(term_list);
-					}
-					if(found) break;
-					node = g_node_next_sibling(node);
-				}
-				if(found) break;
-				properties_list = g_slist_next(properties_list);
-			}
-			
-			mapping = (WNISynonymMapping*) ((GNode*) properties_list->data)->data;
-
-			mark = highlight_definition(mapping->id, mapping->sense, text_view);
-
-			gtk_text_view_scroll_to_mark(text_view, mark, 0.0, TRUE, 0, 0);
-
-			break;
-		}
-		wni_results = g_slist_next(wni_results);
-	}
-}
-*/
-
 static void relative_selection_changed(GtkTreeView *tree_view, gpointer user_data)
 {
 	GtkTreeIter iter = {0};
@@ -1959,8 +2172,8 @@ static void create_text_view_tags(GtkBuilder *gui_builder)
 	gtk_text_buffer_create_tag(buffer, TAG_LEMMA, "weight", PANGO_WEIGHT_BOLD, NULL);
 	gtk_text_buffer_create_tag(buffer, TAG_POS, "foreground", "red", "style", PANGO_STYLE_ITALIC, NULL);
 	gtk_text_buffer_create_tag(buffer, TAG_COUNTER, "left_margin", 15, "weight", PANGO_WEIGHT_BOLD, "foreground", "gray", NULL);
-	gtk_text_buffer_create_tag(buffer, TAG_EXAMPLE, "foreground", "blue", "font", "FreeSerif Italic 11", "left_margin", 45, NULL);
-	gtk_text_buffer_create_tag(buffer, TAG_SUGGEST, "foreground", "DarkGreen", "weight", PANGO_WEIGHT_SEMIBOLD, NULL);
+	gtk_text_buffer_create_tag(buffer, TAG_EXAMPLE, "foreground", "blue", "font", "Serif Italic 10", "left_margin", 45, NULL);
+	gtk_text_buffer_create_tag(buffer, TAG_MATCH, "foreground", "DarkGreen", "weight", PANGO_WEIGHT_SEMIBOLD, NULL);
 	gtk_text_buffer_create_tag(buffer, TAG_SUGGESTION, "foreground", "blue" , "left_margin", 25, NULL);
 	gtk_text_buffer_create_tag(buffer, TAG_HIGHLIGHT, "background", "black", "foreground", "white", NULL);
 	
@@ -2176,6 +2389,12 @@ static void about_url_hook(GtkAboutDialog *about_dialog, const gchar *link, gpoi
 
 static void destructor(Display *dpy, guint keyval, GtkBuilder *gui_builder)
 {
+	if(wordnet_terms)
+	{
+	        g_string_free(wordnet_terms, TRUE);
+	        wordnet_terms = NULL;
+	}
+
 	if(last_search)
 		g_free(last_search);
 
@@ -2204,7 +2423,6 @@ static void destructor(Display *dpy, guint keyval, GtkBuilder *gui_builder)
 	if(notifier)
 	{
 		notify_notification_close(notifier, NULL);
-		notify_notification_clear_actions(notifier);
 
 		// Its better to be safe than to be sorry
 		// some times, notifier becomes NULL
@@ -2217,6 +2435,93 @@ static void destructor(Display *dpy, guint keyval, GtkBuilder *gui_builder)
 #endif
 }
 
+static gboolean wordnet_terms_load(GtkBuilder *gui_builder)
+{
+	gchar *index_file_path = NULL;
+	gchar *contents = NULL;
+	GString *word, *prev_word = NULL;
+	gchar ch = 0;
+	guint32 i = 0;
+	guint32 count = 0;
+	gboolean ret_val = FALSE;
+	GtkWindow *window = GTK_WINDOW(gtk_builder_get_object(gui_builder, WINDOW_MAIN));
+	GtkStatusbar *status_bar = GTK_STATUSBAR(gtk_builder_get_object(gui_builder, STATUSBAR));
+	gchar status_msg[MAX_STATUS_MSG] = "";
+
+	if(SetSearchdir())
+	{
+		index_file_path = g_strdup_printf(SENSEIDXFILE, SetSearchdir());
+		
+		if(g_file_get_contents(index_file_path, &contents, NULL, NULL))
+		{
+			word = g_string_new("");
+			prev_word = g_string_new("");
+			wordnet_terms = g_string_new("");
+
+			// set the loading message on status bar
+			gtk_statusbar_pop(status_bar, status_msg_context_id);
+			g_snprintf(status_msg, MAX_STATUS_MSG, STR_STATUS_INDEXING);
+			status_msg_context_id = gtk_statusbar_get_context_id(status_bar, STATUS_DESC_LOADING_INDEX);
+			gtk_statusbar_push(status_bar, status_msg_context_id, status_msg);
+
+			// if visible, repaint the statusbar after setting the status message, for it to get reflected
+			if(GTK_WIDGET_VISIBLE(window))
+				gdk_window_process_updates(((GtkWidget*)status_bar)->window, FALSE);
+
+			do
+			{
+				ch = contents[i++];
+				
+				if(ch == '_') ch = ' ';
+
+				if(ch == '%')
+				{
+					for(; ((contents[i] != '\n') && (contents[i] != '\0')); i++);
+
+					ch = contents[i++];
+					word = g_string_append_c(word, ch);
+
+					if(!g_string_equal(prev_word, word))
+					{
+						wordnet_terms = g_string_append(wordnet_terms, word->str);
+						g_string_erase(prev_word, 0, prev_word->len);
+						g_string_append(prev_word, word->str);
+
+						count++;
+					}
+					word = g_string_erase(word, 0, word->len);
+				}
+				else
+				{
+					word = g_string_append_c(word, ch);
+				}
+
+			} while(ch != '\0');
+
+			// put up the EOF char in place of the lastly appended '\n'
+			// Note: since it's a GString, after this '\n' there will be
+			// a default '\0' managed by GLib; GString->len gives you
+			// the length without counting this char.
+			wordnet_terms->str[wordnet_terms->len - 1] = ch;
+			
+			G_DEBUG("Total Dict. Terms Loaded: %d\n", ++count);
+
+			g_free(contents);
+			g_string_free(word, TRUE);
+			g_string_free(prev_word, TRUE);
+			
+			ret_val = TRUE;
+
+			// clear the loading message from status bar
+			gtk_statusbar_pop(status_bar, status_msg_context_id);
+		}
+		
+		g_free(index_file_path);
+	}
+	
+	return ret_val;
+}
+
 int main(int argc, char *argv[])
 {
 	Display *dpy = NULL;
@@ -2240,7 +2545,6 @@ int main(int argc, char *argv[])
 
 	GtkWidget *msg_dialog = NULL;
 
-
 	if(!g_thread_supported())
 	{
 		G_DEBUG("Calling g_thread_init()!\n");
@@ -2418,19 +2722,23 @@ int main(int argc, char *argv[])
 					
 					mod_suggest = suggestions_init();
 
-					// set via GUI.ui
+					// this set via GUI.ui now, so this not required
 					//gtk_window_set_startup_id(GTK_WINDOW(window), APP_ID);
 
-					// To show or not to show? on startup - better show it the very first time, when the user sets the option
-					// start up hided, then don't show it
+					// if its a first run (anew/updated) save the preferences now
 					if(first_run)
-						// since its a first run (anew/updated) save the preferences once
 						save_preferences();
 
+					// show the window if it's a first run or a hot key couldn't be set
+					// if the window is not shown, set notify the startup is complete
 					if(first_run || x_error)
 						gtk_widget_show_all(window);
+					else
+						gdk_notify_startup_complete();
+
+					// index all wordnet terms from the index.sense onto memory
+					wordnet_terms_load(gui_builder);
 
-					
 					gtk_main();
 
 					// since every setting change will call save prefs.
